@@ -1,5 +1,4 @@
 pub mod effects;
-mod ffi;
 mod lists;
 mod logics;
 mod maps;
@@ -7,22 +6,31 @@ mod math;
 pub mod meta;
 mod records;
 mod refs;
-mod regexes;
 mod sets;
 mod strings;
 mod syntax;
 
-use crate::data::json;
-use crate::primes::{Calcit, CalcitItems, CalcitScope};
-use crate::program::ProgramCodeData;
+use std::collections::HashMap;
+use std::sync::{Arc, RwLock};
+
+use crate::call_stack::CallStackList;
+use crate::primes::{Calcit, CalcitErr, CalcitItems, CalcitScope, CalcitSyntax};
+
+pub type FnType = fn(xs: &CalcitItems, call_stack: &CallStackList) -> Result<Calcit, CalcitErr>;
+pub type SyntaxType = fn(expr: &CalcitItems, scope: &CalcitScope, file_ns: &str) -> Result<Calcit, CalcitErr>;
+
+lazy_static! {
+  static ref IMPORTED_PROCS: RwLock<HashMap<Arc<str>, FnType>> = RwLock::new(HashMap::new());
+}
 
 pub fn is_proc_name(s: &str) -> bool {
-  matches!(
+  let builtin = matches!(
     s,
     // meta
     "type-of"
       | "recur"
       | "format-to-lisp"
+      | "format-to-cirru"
       | "gensym"
       | "&reset-gensym-index!"
       | "&get-calcit-running-mode"
@@ -37,44 +45,19 @@ pub fn is_proc_name(s: &str) -> bool {
       | "&tuple:assoc"
       // effects
       | "&display-stack"
-      | "echo"
-      | "println" // alias for echo
-      | "echo-values"
       | "raise"
       | "quit!"
       | "get-env"
       | "&get-calcit-backend"
       | "read-file"
       | "write-file"
-      // ffi
-      | "&ffi-message"
-      | "&call-dylib:str->str"
-      | "&call-dylib:str->unit"
-      | "&call-dylib:str:str->str"
-      | "&call-dylib:str->bool"
-      | "&call-dylib:->str"
-      | "&call-dylib:str->vec-str"
-      | "&call-dylib:vec-str->tuple-str2"
-      | "&call-dylib:str-vec-str->tuple-str2"
-      | "&call-dylib:cirru->str"
-      | "&call-dylib:str-i64->i64"
       // external format
       | "parse-cirru"
       | "format-cirru"
       | "parse-cirru-edn"
       | "format-cirru-edn"
-      | "parse-json"
-      | "stringify-json"
-      // regex
-      | "re-matches"
-      | "re-find"
-      | "re-find-index"
-      | "re-find-all"
       // time
       | "cpu-time"
-      | "format-time"
-      | "parse-time"
-      | "get-time!"
       // logics
       | "&="
       | "&<"
@@ -87,8 +70,6 @@ pub fn is_proc_name(s: &str) -> bool {
       | "&*"
       | "&/"
       | "round"
-      | "rand"
-      | "rand-int"
       | "floor"
       | "sin"
       | "cos"
@@ -134,6 +115,10 @@ pub fn is_proc_name(s: &str) -> bool {
       | "prepend"
       | "butlast"
       | "range"
+      | "sort"
+      | "foldl"
+      | "foldl-shortcut"
+      | "foldr-shortcut"
       | "&list:reverse"
       | "&list:concat"
       | "&list:count"
@@ -195,19 +180,38 @@ pub fn is_proc_name(s: &str) -> bool {
       | "&record:to-map"
       | "&record:count"
       | "&record:contains?"
-      | "&record:nth"
       | "&record:get"
       | "&record:assoc"
       | "&record:extend-as"
-  )
+  );
+  if builtin {
+    true
+  } else {
+    let ps = IMPORTED_PROCS.read().unwrap();
+    ps.contains_key(s)
+  }
 }
 
-pub fn handle_proc(name: &str, args: &CalcitItems) -> Result<Calcit, String> {
+/// make sure that stack information attached in errors from procs
+pub fn handle_proc(name: &str, args: &CalcitItems, call_stack: &CallStackList) -> Result<Calcit, CalcitErr> {
+  handle_proc_internal(name, args, call_stack).map_err(|e| {
+    if e.stack.is_empty() {
+      let mut e2 = e.to_owned();
+      e2.stack = call_stack.to_owned();
+      e2
+    } else {
+      e
+    }
+  })
+}
+
+fn handle_proc_internal(name: &str, args: &CalcitItems, call_stack: &CallStackList) -> Result<Calcit, CalcitErr> {
   match name {
     // meta
     "type-of" => meta::type_of(args),
     "recur" => meta::recur(args),
     "format-to-lisp" => meta::format_to_lisp(args),
+    "format-to-cirru" => meta::format_to_cirru(args),
     "gensym" => meta::gensym(args),
     "&reset-gensym-index!" => meta::reset_gensym_index(args),
     "&get-calcit-running-mode" => effects::calcit_running_mode(args),
@@ -221,45 +225,20 @@ pub fn handle_proc(name: &str, args: &CalcitItems) -> Result<Calcit, String> {
     "&tuple:nth" => meta::tuple_nth(args),
     "&tuple:assoc" => meta::assoc(args),
     // effects
-    "&display-stack" => meta::display_stack(args),
-    "echo" => effects::echo(args),
-    "println" => effects::echo(args), // alias
-    "echo-values" => effects::echo_values(args),
+    "&display-stack" => meta::display_stack(args, call_stack),
     "raise" => effects::raise(args),
     "quit!" => effects::quit(args),
     "get-env" => effects::get_env(args),
     "&get-calcit-backend" => effects::call_get_calcit_backend(args),
     "read-file" => effects::read_file(args),
     "write-file" => effects::write_file(args),
-    // ffi
-    "&ffi-message" => ffi::ffi_message(args),
-    "&call-dylib:str->str" => ffi::call_dylib_str_to_str(args),
-    "&call-dylib:str->unit" => ffi::call_dylib_str_to_unit(args),
-    "&call-dylib:str:str->str" => ffi::call_dylib_str_str_to_str(args),
-    "&call-dylib:str->bool" => ffi::call_dylib_str_to_bool(args),
-    "&call-dylib:->str" => ffi::call_dylib_to_str(args),
-    "&call-dylib:str->vec-str" => ffi::call_dylib_str_to_vec_str(args),
-    "&call-dylib:vec-str->tuple-str2" => ffi::call_dylib_vec_str_to_tuple_str2(args),
-    "&call-dylib:str-vec-str->tuple-str2" => ffi::call_dylib_str_vec_str_to_tuple_str2(args),
-    "&call-dylib:cirru->str" => ffi::call_dylib_cirru_to_str(args),
-    "&call-dylib:str-i64->i64" => ffi::call_dylib_str_i64_to_i64(args),
     // external data format
     "parse-cirru" => meta::parse_cirru(args),
     "format-cirru" => meta::format_cirru(args),
     "parse-cirru-edn" => meta::parse_cirru_edn(args),
     "format-cirru-edn" => meta::format_cirru_edn(args),
-    "parse-json" => json::parse_json(args),
-    "stringify-json" => json::stringify_json(args),
     // time
     "cpu-time" => effects::cpu_time(args),
-    "parse-time" => effects::parse_time(args),
-    "format-time" => effects::format_time(args),
-    "get-time!" => effects::now_bang(args),
-    // regex
-    "re-matches" => regexes::re_matches(args),
-    "re-find" => regexes::re_find(args),
-    "re-find-index" => regexes::re_find_index(args),
-    "re-find-all" => regexes::re_find_all(args),
     // logics
     "&=" => logics::binary_equal(args),
     "&<" => logics::binary_less(args),
@@ -272,8 +251,6 @@ pub fn handle_proc(name: &str, args: &CalcitItems) -> Result<Calcit, String> {
     "&-" => math::binary_minus(args),
     "&*" => math::binary_multiply(args),
     "&/" => math::binary_divide(args),
-    "rand" => math::rand(args),
-    "rand-int" => math::rand_int(args),
     "floor" => math::floor(args),
     "sin" => math::sin(args),
     "cos" => math::cos(args),
@@ -321,6 +298,10 @@ pub fn handle_proc(name: &str, args: &CalcitItems) -> Result<Calcit, String> {
     "butlast" => lists::butlast(args),
     "&list:concat" => lists::concat(args),
     "range" => lists::range(args),
+    "sort" => lists::sort(args, call_stack),
+    "foldl" => lists::foldl(args, call_stack),
+    "foldl-shortcut" => lists::foldl_shortcut(args, call_stack),
+    "foldr-shortcut" => lists::foldr_shortcut(args, call_stack),
     "&list:reverse" => lists::reverse(args),
     "&list:slice" => lists::slice(args),
     "&list:assoc-before" => lists::assoc_before(args),
@@ -380,67 +361,51 @@ pub fn handle_proc(name: &str, args: &CalcitItems) -> Result<Calcit, String> {
     "&record:matches?" => records::matches(args),
     "&record:count" => records::count(args),
     "&record:contains?" => records::contains_ques(args),
-    "&record:nth" => records::nth(args),
     "&record:get" => records::get(args),
     "&record:assoc" => records::assoc(args),
     "&record:extend-as" => records::extend_as(args),
-    a => Err(format!("No such proc: {}", a)),
+    a => {
+      let ps = IMPORTED_PROCS.read().unwrap();
+      if ps.contains_key(name) {
+        let f = ps[name];
+        f(args, call_stack)
+      } else {
+        Err(CalcitErr::use_msg_stack(format!("No such proc: {}", a), call_stack))
+      }
+    }
   }
 }
 
-pub fn is_syntax_name(s: &str) -> bool {
-  matches!(
-    s,
-    "defn"
-      | "defmacro"
-      | "if"
-      | "&let"
-      | "quote"
-      | "quasiquote"
-      | "eval"
-      | "macroexpand"
-      | "macroexpand-1"
-      | "macroexpand-all"
-      | "foldl" // for performance
-      | "foldl-shortcut" // for performance
-      | "foldr-shortcut" // for performance
-      | "try"
-      | "sort" // TODO need better solution
-      | "defatom"
-      | "reset!"
-      | "hint-fn"
-  )
+/// inject into procs
+pub fn register_import_proc(name: &str, f: FnType) {
+  let mut ps = IMPORTED_PROCS.write().unwrap();
+  (*ps).insert(name.to_owned().into(), f);
 }
 
 pub fn handle_syntax(
-  name: &str,
+  name: &CalcitSyntax,
   nodes: &CalcitItems,
   scope: &CalcitScope,
-  file_ns: &str,
-  program: &ProgramCodeData,
-) -> Result<Calcit, String> {
+  file_ns: Arc<str>,
+  call_stack: &CallStackList,
+) -> Result<Calcit, CalcitErr> {
   match name {
-    "defn" => syntax::defn(nodes, scope, file_ns, program),
-    "eval" => syntax::eval(nodes, scope, file_ns, program),
-    "defmacro" => syntax::defmacro(nodes, scope, file_ns, program),
-    "quote" => syntax::quote(nodes, scope, file_ns, program),
-    "quasiquote" => syntax::quasiquote(nodes, scope, file_ns, program),
-    "if" => syntax::syntax_if(nodes, scope, file_ns, program),
-    "&let" => syntax::syntax_let(nodes, scope, file_ns, program),
-    "foldl" => lists::foldl(nodes, scope, file_ns, program),
-    "foldl-shortcut" => lists::foldl_shortcut(nodes, scope, file_ns, program),
-    "foldr-shortcut" => lists::foldr_shortcut(nodes, scope, file_ns, program),
-    "macroexpand" => syntax::macroexpand(nodes, scope, file_ns, program),
-    "macroexpand-1" => syntax::macroexpand_1(nodes, scope, file_ns, program),
-    "macroexpand-all" => syntax::macroexpand_all(nodes, scope, file_ns, program),
-    "try" => syntax::call_try(nodes, scope, file_ns, program),
-    "sort" => lists::sort(nodes, scope, file_ns, program),
+    CalcitSyntax::Defn => syntax::defn(nodes, scope, file_ns),
+    CalcitSyntax::Eval => syntax::eval(nodes, scope, file_ns, call_stack),
+    CalcitSyntax::Defmacro => syntax::defmacro(nodes, scope, file_ns),
+    CalcitSyntax::Quote => syntax::quote(nodes, scope, file_ns),
+    CalcitSyntax::Quasiquote => syntax::quasiquote(nodes, scope, file_ns, call_stack),
+    CalcitSyntax::If => syntax::syntax_if(nodes, scope, file_ns, call_stack),
+    CalcitSyntax::CoreLet => syntax::syntax_let(nodes, scope, file_ns, call_stack),
+    CalcitSyntax::Macroexpand => syntax::macroexpand(nodes, scope, file_ns, call_stack),
+    CalcitSyntax::Macroexpand1 => syntax::macroexpand_1(nodes, scope, file_ns, call_stack),
+    CalcitSyntax::MacroexpandAll => syntax::macroexpand_all(nodes, scope, file_ns, call_stack),
+    CalcitSyntax::Try => syntax::call_try(nodes, scope, file_ns, call_stack),
     // "define reference" although it uses a confusing name "atom"
-    "defatom" => refs::defatom(nodes, scope, file_ns, program),
-    "reset!" => refs::reset_bang(nodes, scope, file_ns, program),
+    CalcitSyntax::Defatom => refs::defatom(nodes, scope, file_ns, call_stack),
+    CalcitSyntax::Reset => refs::reset_bang(nodes, scope, file_ns, call_stack),
     // different behavoirs, in Rust interpreter it's nil, in js codegen it's nothing
-    "hint-fn" => meta::no_op(),
-    a => Err(format!("TODO syntax: {}", a)),
+    CalcitSyntax::HintFn => meta::no_op(),
   }
 }
 
